package com.reactlibrary;

import android.app.Activity;
import android.app.Dialog;
import android.graphics.Color;
import android.graphics.Typeface;
import android.graphics.drawable.ColorDrawable;
import android.graphics.drawable.GradientDrawable;
import android.os.Handler;
import android.os.Looper;
import android.util.TypedValue;
import android.view.Gravity;
import android.view.View;
import android.view.WindowManager;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReadableMap;

public class CustomLayoutModule extends ReactContextBaseJavaModule {

    private final ReactApplicationContext reactContext;
    Dialog dialog;
    
    public CustomLayoutModule(ReactApplicationContext reactContext) {
        super(reactContext);
        this.reactContext = reactContext;
    }

    @Override
    public String getName() {
        return "CustomLayout";
    }

    @ReactMethod
    public void showDialog(ReadableMap data, final Callback onClose) {
        Activity activity = getCurrentActivity();

        // LAYOUT VARIABLE
        final int duration = data.hasKey("duration") ? data.getInt("duration") : 3500;
        String position_text = "bottom";
        String bgColor = "#000000";
        int radius = 20;
        int padding_left = 25;
        int padding_top = 35;
        int padding_right = 25;
        int padding_bottom = 35;
        int margin_left = 35;
        int margin_top = 0;
        int margin_right = 35;
        int margin_bottom = 50;
        if(data.hasKey("layout_style")){
            ReadableMap layout_style = data.hasKey("layout_style") ? data.getMap("layout_style") : null;
            if(layout_style.hasKey("position")) position_text = layout_style.getString("position");
            if(layout_style.hasKey("bgColor")) bgColor = layout_style.getString("bgColor");
            if(layout_style.hasKey("radius")) radius = layout_style.getInt("radius");
            if(layout_style.hasKey("padding_left")) padding_left = layout_style.getInt("padding_left");
            if(layout_style.hasKey("padding_top")) padding_top = layout_style.getInt("padding_top");
            if(layout_style.hasKey("padding_right")) padding_right = layout_style.getInt("padding_right");
            if(layout_style.hasKey("padding_bottom")) padding_bottom = layout_style.getInt("padding_bottom");
            if(layout_style.hasKey("margin_left")) margin_left = layout_style.getInt("margin_left");
            if(layout_style.hasKey("margin_top")) margin_top = layout_style.getInt("margin_top");
            if(layout_style.hasKey("margin_right")) margin_right = layout_style.getInt("margin_right");
            if(layout_style.hasKey("margin_bottom")) margin_bottom = layout_style.getInt("margin_bottom");
        }

        //TEXT LEFT VARIABLE
        String text_left = data.hasKey("text_left") ? data.getString("text_left") : "";
        String text_left_color = "#FFFFFF";
        String text_left_textTransform_text = "";
        int text_left_fontSize = 14;
        boolean text_left_uppercase = false;
        if(data.hasKey("text_left_style")){
            ReadableMap text_left_style = data.hasKey("text_left_style") ? data.getMap("text_left_style") : null;
            if(text_left_style.hasKey("textColor")) text_left_color = text_left_style.getString("textColor");
            if(text_left_style.hasKey("fontSize")) text_left_fontSize = text_left_style.getInt("fontSize");
            if(text_left_style.hasKey("textTransform")) text_left_textTransform_text = text_left_style.getString("textTransform");
            if(text_left_style.hasKey("uppercase")) text_left_uppercase = text_left_style.getBoolean("uppercase");
        }

        //TEXT RIGHT VARIABLE
        String text_right = data.hasKey("text_right") ? data.getString("text_right") : "";
        String text_right_color = "#FFFFFF";
        String text_right_textTransform_text = "";
        int text_right_fontSize = 14;
        int text_right_minWidth = 200;
        boolean text_right_uppercase = false;
        if(data.hasKey("text_right_style")){
            ReadableMap text_right_style = data.hasKey("text_right_style") ? data.getMap("text_right_style") : null;
            if(text_right_style.hasKey("textColor")) text_right_color = text_right_style.getString("textColor");
            if(text_right_style.hasKey("fontSize")) text_right_fontSize = text_right_style.getInt("fontSize");
            if(text_right_style.hasKey("minWidth")) text_right_minWidth = text_right_style.getInt("minWidth");
            if(text_right_style.hasKey("textTransform")) text_right_textTransform_text = text_right_style.getString("textTransform");
            if(text_right_style.hasKey("uppercase")) text_right_uppercase = text_right_style.getBoolean("uppercase");
        }

        try {
            dialog = new Dialog(activity);

            int position;
            if(position_text.equals("top")){
                position = Gravity.TOP;
            }else if(position_text.equals("center")){
                position = Gravity.CENTER;
            }else{
                position = Gravity.BOTTOM;
            }

            int text_left_textTransform;
            if(text_left_textTransform_text.equals("bold")){
                text_left_textTransform = Typeface.BOLD;
            }else if(text_left_textTransform_text.equals("italic")){
                text_left_textTransform = Typeface.ITALIC;
            }else if(text_left_textTransform_text.equals("bold_italic")){
                text_left_textTransform = Typeface.BOLD_ITALIC;
            }else{
                text_left_textTransform = Typeface.NORMAL;
            }

            int text_right_textTransform;
            if(text_right_textTransform_text.equals("bold")){
                text_right_textTransform = Typeface.BOLD;
            }else if(text_right_textTransform_text.equals("italic")){
                text_right_textTransform = Typeface.ITALIC;
            }else if(text_right_textTransform_text.equals("bold_italic")){
                text_right_textTransform = Typeface.BOLD_ITALIC;
            }else{
                text_right_textTransform = Typeface.NORMAL;
            }


//        LAYOUT OUTER
            LinearLayout toast_view_outer = new LinearLayout(activity);
            LinearLayout.LayoutParams toast_view_outer_params = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
            toast_view_outer.setBackgroundColor(Color.TRANSPARENT);
            toast_view_outer.setOrientation(LinearLayout.VERTICAL);
            toast_view_outer.setLayoutParams(toast_view_outer_params);

//        INNER LAYOUT
            LinearLayout toast_view_inner = new LinearLayout(activity);
            LinearLayout.LayoutParams toast_view_inner_params = new LinearLayout.LayoutParams(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
            toast_view_inner_params.setMargins(margin_left, margin_top, margin_right, margin_bottom);
            toast_view_inner.setPadding(padding_left, padding_top, padding_right, padding_bottom);
            toast_view_inner.setOrientation(LinearLayout.HORIZONTAL);
            toast_view_inner.setLayoutParams(toast_view_inner_params);
            toast_view_outer.addView(toast_view_inner);

//        LINEAR LAYOUT BACKGROUND
            GradientDrawable shape =  new GradientDrawable();
            shape.setColor(Color.parseColor(bgColor));
            shape.setCornerRadius(radius);
            toast_view_inner.setBackground(shape);

//        TEXT TOAST LEFT
            TextView text_left_tv = new TextView(activity);
            text_left_tv.setLayoutParams(new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT, 1f));
            text_left_tv.setGravity(Gravity.START);
            text_left_tv.setTextColor(Color.parseColor(text_left_color));
            text_left_tv.setText(text_left);
            text_left_tv.setTextSize(TypedValue.COMPLEX_UNIT_SP, text_left_fontSize);
            text_left_tv.setAllCaps(text_left_uppercase);
            text_left_tv.setTypeface(null, text_left_textTransform);
            toast_view_inner.addView(text_left_tv);

//        TEXT TOAST RIGHT
            if(text_right != ""){
                TextView text_right_tv = new TextView(activity);
                text_right_tv.setLayoutParams(new LinearLayout.LayoutParams(LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT, 1f));
                text_right_tv.setGravity(Gravity.END);
                text_right_tv.setTextColor(Color.parseColor(text_right_color));
                text_right_tv.setText(text_right);
                text_right_tv.setTextSize(TypedValue.COMPLEX_UNIT_SP, text_right_fontSize);
                text_right_tv.setMinWidth(text_right_minWidth);
                text_right_tv.setAllCaps(text_right_uppercase);
                text_right_tv.setTypeface(null, text_right_textTransform);
                text_right_tv.setOnClickListener(new View.OnClickListener() {
                    @Override
                    public void onClick(View view) {
                        if(onClose != null){
                            onClose.invoke();
                        }
                        dialog.dismiss();
                    }
                });
                toast_view_inner.addView(text_right_tv);
            }

//        TOAST CONFIGURATION
            dialog.dismiss();
            dialog.setContentView(toast_view_outer);
            dialog.getWindow().setGravity(position);
            dialog.getWindow().setLayout(LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.WRAP_CONTENT);
            dialog.getWindow().setBackgroundDrawable(new ColorDrawable(Color.TRANSPARENT));
            dialog.getWindow().clearFlags(WindowManager.LayoutParams.FLAG_DIM_BEHIND);
            dialog.setCanceledOnTouchOutside(false);
            dialog.show();

            closeDialog(duration);
        }
        catch(Exception e) {
            System.out.println("ERROR : " + e.getMessage());
            Toast.makeText(activity, e.getMessage(), Toast.LENGTH_LONG).show();
        }
    }

    public void closeDialog(int duration){
        new Handler(Looper.getMainLooper()).postDelayed(new Runnable() {
            @Override
            public void run() {
                dialog.dismiss();
            }
        }, duration);
    }

}
